﻿using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;

namespace TestProject
{
    /// <summary>
    /// 메인 윈도우
    /// </summary>
    public partial class MainWindow : Window
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 베이스 포인트
        /// </summary>
        private Point basePoint = new Point(0.0, 0.0);

        /// <summary>
        /// 엘리먼트 내 마우스 포인트
        /// </summary>
        private Point mousePointInElement;

        /// <summary>
        /// 델타 X
        /// </summary>
        private double deltaX = 0.0;

        /// <summary>
        /// 델타 Y
        /// </summary>
        private double deltaY = 0.0;

        /// <summary>
        /// 드래그 여부
        /// </summary>
        private bool isDragging = false;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainWindow()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainWindow()
        {
            InitializeComponent();

            this.label.MouseDown += label_MouseDown;
            this.label.MouseMove += label_MouseMove;
            this.label.MouseUp   += label_MouseUp;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 레이블 마우스 DOWN 처리하기 - label_MouseDown(sender, e)

        /// <summary>
        /// 레이블 마우스 DOWN 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void label_MouseDown(object sender, MouseButtonEventArgs e)
        {
            Label label = e.Source as Label;

            if(label != null)
            {
                label.CaptureMouse();

                this.isDragging = true;

                this.mousePointInElement = e.GetPosition(label);
            }
        }

        #endregion
        #region 레이블 마우스 이동시 처리하기 - label_MouseMove(sender, e)

        /// <summary>
        /// 레이블 마우스 이동시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void label_MouseMove(object sender, MouseEventArgs e)
        {
            if(this.isDragging)
            {
                Point mousePoint = e.GetPosition(null);

                this.deltaX = mousePoint.X - this.basePoint.X - this.mousePointInElement.X;
                this.deltaY = mousePoint.Y - this.basePoint.Y - this.mousePointInElement.Y;

                Canvas.SetLeft(this.label, this.basePoint.X + this.deltaX);
                Canvas.SetTop (this.label, this.basePoint.Y + this.deltaY);
            }
        }

        #endregion
        #region 레이블 마우스 UP 처리하기 - label_MouseUp(sender, e)

        /// <summary>
        /// 레이블 마우스 UP 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void label_MouseUp(object sender, MouseButtonEventArgs e)
        {
            Label label = e.Source as Label;

            if(label != null)
            {
                label.ReleaseMouseCapture();

                this.basePoint.X += deltaX;
                this.basePoint.Y += deltaY;

                this.deltaX = 0.0;
                this.deltaY = 0.0;

                this.isDragging = false;
            }
        }

        #endregion
    }
}